@@ -11,7 +11,17 @@ import h5py
 from subprocess import check_call, Popen, PIPE
 
 
-def cp(file1,file2):
+def cp(file1,file2 = None):
+    if file2 is None:
+        ## check if a single string was given in, separated by a space
+        temp_f_list = file1.split(" ")
+        if len(temp_f_list)==2:
+            file1 = temp_f_list[0]
+            file2 = temp_f_list[1]
+        else:
+            print("\n\nsomething wrong with the cp syntax!\n\n")
+            print("don't know how to interpret:",file1)
+            sys.exit()
     with open(file1, 'rb') as f_in:
         with open(file2, 'wb') as f_out:
             shutil.copyfileobj(f_in, f_out)
@@ -167,10 +177,7 @@ def make_table(lines, delim, range_min=0, num_convert = float):
 
 
 def get_file_path(in_path):
-    in_path = in_path.split('/')
-    in_path = in_path[:-1]
-    in_path = '/'.join(in_path)
-    return in_path + '/'
+    return os.path.dirname(in_path)+os.sep
 
 
 def read_table(file, sep='\t', num_convert = float):
@@ -508,4 +515,4 @@ def read_tsv(in_file):
     for i in range(1,len(in_table)):
         genes.append(in_table[i,0])
         in_mat_num[i-1,:] = in_table[i,1:]
-    return(in_mat, genes, cells)
+    return(in_mat_num, genes, cells)
