@@ -8,7 +8,7 @@ from time import sleep
 from scipy.stats import spearmanr
 from time import time
 import fileinput,pickle
-from scipy.stats import gaussian_kde
+from scipy.stats import gaussian_kde, rankdata
 import matplotlib.pyplot as plt
 import matplotlib.cm as cm
 import seaborn as sns
@@ -17,12 +17,11 @@ from copy import deepcopy
 try:
     from pyminer.common_functions import *
     from pyminer.pyminer_common_stats_functions import *
-    from pyminer.pyminer_common_stats_functions import no_p_spear, get_Z_cutoff
+    from pyminer.pyminer_common_stats_functions import no_p_spear, get_Z_cutoff, get_empiric_FPR
 except:
     from common_functions import *
     from pyminer_common_stats_functions import *
-    from pyminer_common_stats_functions import no_p_spear, get_Z_cutoff
-print(no_p_spear)
+    from pyminer_common_stats_functions import no_p_spear, get_Z_cutoff, get_empiric_FPR
 
 ##################################################################################
 
@@ -238,7 +237,7 @@ if args.rho_dict_dir==None:
     rho_dict_dir = temp_dir+'/rho_dicts'
 else:
     rho_dict_dir = args.rho_dict_dir
-cmd('mkdir '+rho_dict_dir)
+process_dir(rho_dict_dir)
 #########################################################
 
 #output_lines = [["X_var","Y_var","Spearman_p_val","Spearman_rho"]]
@@ -258,7 +257,7 @@ if args.hdf5_out:
     ## make the hdf5 output file
     hdf5_spear_out_file = rho_dict_dir+"/spearman.hdf5"
     print(hdf5_spear_out_file)
-    #cmd('rm '+hdf5_spear_out_file)
+    #rm(hdf5_spear_out_file)
     
     spear_f = h5py.File(hdf5_spear_out_file, "a")
     try:
@@ -268,17 +267,25 @@ if args.hdf5_out:
 
 
 
-print(np.shape(infile))
-
-
-print(infile)
-
-print(bins)
+print("input shape:",np.shape(infile))
 
+print("bins:",bins)
+###################################################################
+## this variable is if we're doing a global negative rho cutoff, or a variable one based on sparsity
+single_cutoff = True
+if not single_cutoff:
+    ## This var is for keeping track of the sparsity of every gene
+    gene_num_zero = np.zeros((total_vars))
+    gene_non_zero_mean = np.zeros((total_vars))
+    num_cells = infile.shape[1]
 
 start = time.time()
 print("finding block...")
 for i in range(0,(len(bins)-1)):
+    temp_first = infile[bins[i]:bins[i+1],]
+    if not single_cutoff:
+        gene_num_zero[bins[i]:bins[i+1]] = np.sum(temp_first==0,axis=1)
+        gene_non_zero_mean[bins[i]:bins[i+1]] = np.sum(temp_first,axis=1) / (num_cells - gene_num_zero[bins[i]:bins[i+1]])
     for j in range(i,(len(bins)-1)):
         if (i!=j) or (len(bins) == 2):#True:
             out_rho_dict = rho_dict_dir+"/rho_"+str(bins[i])+"_to_"+str(bins[i+1])+"_vs_"+str(bins[j])+"_to_"+str(bins[j+1])+".pkl"
@@ -305,7 +312,7 @@ for i in range(0,(len(bins)-1)):
                 ## find the spearman correlations
 
                 print('working on',bins[i],bins[i+1],'vs',bins[j],bins[j+1])
-                r=no_p_spear(infile[bins[i]:bins[i+1],],infile[bins[j]:bins[j+1],], axis = 1, prop=args.prop)
+                r=no_p_spear(temp_first,infile[bins[j]:bins[j+1],], axis = 1, prop=args.prop)
                 
                 ## if we're not making the spearman correlation matrix into an hdf5 file - save the dictionary:
                 if not args.hdf5_out:
@@ -313,7 +320,7 @@ for i in range(0,(len(bins)-1)):
 
                 ## if we are making the spearman correlation matrix an hdf5 file, we've got some work to do:
                 else:
-                    print(np.shape(r))
+                    #print(np.shape(r))
                     if np.shape(r)[0] == 2*bin_size:
                         #print('is symmetric:',r[bin_size:,bin_size:] == r[:bin_size,:bin_size])
                         if len(bins)==2:#False#np.shape(r)[0]<=bin_size:
@@ -321,7 +328,7 @@ for i in range(0,(len(bins)-1)):
                             ## bin size. In this case, the returned matrix will be symmetric in all 4
                             ## quadrants, so we'll reduce it down to just the one
                             r = r[bin_size:,bin_size:]
-                    print(np.shape(r))
+                    #print(np.shape(r))
 
 
                     ## because h5py's fancy indexing doesn't work very well with slicing in 2D...
@@ -365,7 +372,14 @@ for i in range(0,(len(bins)-1)):
 
 end = time.time()
 
-
+######################
+## finish cateloguing the number of zeros
+if not single_cutoff:
+    temp_first = infile[bins[-1]:,]
+    gene_num_zero[bins[-1]:] = np.sum(temp_first==0,axis=1)
+    gene_non_zero_mean[bins[-1]:] = np.sum(temp_first,axis=1) / (num_cells - gene_num_zero[bins[-1]:])
+    gene_percent_express = (num_cells - gene_num_zero)/num_cells
+######################
 
 if args.euclidean_dist:
     from sklearn import metrics
@@ -507,7 +521,7 @@ all_vars_adj_dict = {}
 neg_vars_adj_dict = {}
 total_neg_vars_adj_dict = {}
 for i in range(0,len(IDlist)):
-    ## [ID, sum_negative_rho, count_below_threshold, total_number_negative, (count_below_threshold+1)/(count_above_threshold +1)]
+    ## [ID, sum_negative_rho, count_below_threshold, total_number_negative, (count_below_threshold+1)/(count_above_threshold +1)] # later we also append to this line a T/F pass call
     sum_neg.append([IDlist[i],0,0,0,0])
     all_vars_adj_dict[IDlist[i]] = np.zeros(len(IDlist,),dtype = bool)
     neg_vars_adj_dict[IDlist[i]] = np.zeros(len(IDlist,),dtype = bool)
@@ -551,6 +565,7 @@ def plot_one_density(neg_cor_density_x, neg_cor_density_y,out_file, label = 'neg
     plt.savefig(out_file,
         dpi=600,
         bbox_inches='tight')
+    return()
 
 def plot_one_density_x_cutoff(neg_cor_density_x, neg_cor_density_y,cutoff,out_file,log = False,poisson_mu=None,label = "density"):
     global args
@@ -581,6 +596,7 @@ def plot_one_density_x_cutoff(neg_cor_density_x, neg_cor_density_y,cutoff,out_fi
     plt.savefig(out_file,
         dpi=600,
         bbox_inches='tight')
+    return
 
 
 def plot_densities(neg_cor_density_x, neg_cor_density_y,series_2_x,series_2_y,out_file):
@@ -640,18 +656,17 @@ def density_scatter(x,y,out_file,log_density=0,expected_x=[],expected_y=[]):
     except:
         print("error in getting the gaussian KDE")
         return()
-    print(z)
     if log_density:
         while log_density:
             log_density-=1
             z = np.log2(z+1)
             z = z-min(z)
             z = z/max(z)
-            print(z)
     plt.clf()
     fig, ax = plt.subplots()
     ax.scatter(x, y, c=z, s=5, edgecolor='', cmap=plt.cm.jet)
-    if expected_x != [] and expected_y != [] :
+    ###############################################
+    if (expected_x is not []) and (expected_y is not []) :
         ax.plot(expected_x,expected_y,linestyle='-',c='black')
         plt.xlim((-1,max(x)+1))
         plt.ylim((0,max(y)+1))
@@ -660,6 +675,7 @@ def density_scatter(x,y,out_file,log_density=0,expected_x=[],expected_y=[]):
         dpi=600,
         bbox_inches='tight'
         )
+    return
 
 
 ###################
@@ -667,70 +683,31 @@ small_adj_list = []
 
 do_sum_neg = args.sc_clust
 
-def calculate_mad(in_vect, dev_num = 3, mean = False):
-    med = np.median(in_vect)
-    if mean:
-        dev = np.mean(np.abs(in_vect - med))
-    else:
-        dev = np.median(np.abs(in_vect - med))
-    print('median:',med)
-    print('median absolute deviation:',dev)
-    cutoff = med + (dev_num * dev)
-    print('cutoff =',cutoff)
-    return(med,dev,cutoff)
-
-# def get_Z_cutoff(vect,z=4.5,positive = False):
-#     ## this is for a half distribution, so mult by -1 to make it 'normal'
-#     global FPR
-#     vect_original = np.array(deepcopy(vect))
-#     vect = list(vect)
-#     vect += list(np.array(vect)*-1)
-#     vect = np.array(vect)
-#     ## get the mean, & SD
-#     v_mean = np.mean(vect)
-#     v_std = np.std(vect)
-#     cutoff = v_std * z
-#     if not positive:
-#         cutoff *= -1
-#         cutoff = max([-0.9,cutoff])
-#     else:
-#         cutoff = min([0.9,cutoff])
-#     print('cutoff:',cutoff)
-#     if not positive:
-#         num_sig = np.sum(np.array(vect_original<cutoff,dtype = bool))
-#         total_num = vect_original.shape[0]
-#         print(num_sig, "/", total_num, 'significant')
-#         FPR = max([1,total_num])/num_sig
-#         return(cutoff, FPR)
-#     else:
-#         return(cutoff)
-
-
-# def get_FPR_cutoff(negative_control_negative_rho_vect,FPR,positive = False):
-#     ####################################################################
-#     ## figure out the cutoff to use for a given false positive rate
-#     negative_control_negative_rho_vect = negative_control_negative_rho_vect[np.where(abs(negative_control_negative_rho_vect)!=1)[0]]
-#     negative_control_negative_rho_vect = np.sort(negative_control_negative_rho_vect, kind = "mergesort")
-#     if positive:
-#         negative_control_negative_rho_vect = negative_control_negative_rho_vect[::-1]
-#     print("sorting list:")
-#     print(negative_control_negative_rho_vect[:5],'...',negative_control_negative_rho_vect[-5:])
-#     number_of_total_negative_correlations_in_boot = np.shape(negative_control_negative_rho_vect)[0]
-#     print("number_of_total_negative_correlations_in_boot")
-#     print(number_of_total_negative_correlations_in_boot)
-#     print("FPR:",FPR)
-
-#     neg_cutoff_index = int(max([1,number_of_total_negative_correlations_in_boot/FPR])-1)## this is an emperical cutoff of 1/FPR randomly negative rhos
-#     print("cutoff_index")
-#     print(neg_cutoff_index)
-
-#     neg_cutoff = float(negative_control_negative_rho_vect[neg_cutoff_index])
-#     #print(negative_control_negative_rho_vect[:neg_cutoff_index])
-#     print('Rho cutoff:',neg_cutoff)
-#     return(neg_cutoff)
-
-    ##################################################################
 
+def get_rho_cutoff_vect(r, neg_control_sample_idxs, gene_percent_express, out_dir):
+    print("finding dynamic FPR")
+    neg_ctrl_percent_express = gene_percent_express[neg_control_sample_idxs]
+    list_of_percent_express = []
+    list_of_negative_rhos = []
+    for i in range(r.shape[0]):
+        temp_neg_rhos = r[i,r[i,:]<0].tolist()
+        list_of_negative_rhos += temp_neg_rhos
+        list_of_percent_express += np.repeat(neg_ctrl_percent_express[i], len(temp_neg_rhos)).tolist()
+    out_file = os.path.join(out_dir, "percentExpressX_vs_NullNegRhos.png")
+    print("plotting subset percent express vs null negative Rho distributions")
+    ################
+    neg_control_sample_idxs = np.arange(len(list_of_percent_express))
+    ## remove any genes that are zero in all samples
+    np.random.shuffle(neg_control_sample_idxs)
+    subset_idxs = neg_control_sample_idxs[:20000]
+    ################
+    print(len(list_of_percent_express))
+    list_of_percent_express = np.array(list_of_percent_express)
+    list_of_negative_rhos = np.array(list_of_negative_rhos)
+    density_scatter(rankdata(list_of_percent_express[subset_idxs],method="dense"),list_of_negative_rhos[subset_idxs],out_file)
+    sys.exit()
+    return()
+######################################################################
 
 
 if do_sum_neg or args.rho_cutoff==None:
@@ -741,13 +718,26 @@ if do_sum_neg or args.rho_cutoff==None:
     non_zero_idxs = np.where(np.sum(infile,axis=1)>0)[0]
     neg_control_sample_idxs = neg_control_sample_idxs[non_zero_idxs]
     np.random.shuffle(neg_control_sample_idxs)
-    print(np.shape(neg_control_sample_idxs))
+    #print(np.shape(neg_control_sample_idxs))
     bin_size = min([bin_size, np.shape(neg_control_sample_idxs)[0]])
-    print(bin_size)
+    #print(bin_size)
     neg_control_sample_idxs = neg_control_sample_idxs[:bin_size]
+    ##############################################################
+    ## 11/14/2020
+    ## add lowest & highest % expression to make sure we cover the whole range
+    ## & don't cause an error with lowess later
+    if not single_cutoff:
+        percent_express_sorted_idxs = np.argsort(gene_percent_express)
+        highest_percent_idx = percent_express_sorted_idxs[0]
+        lowest_percent_idx = percent_express_sorted_idxs[percent_express_sorted_idxs.shape[0]-1]
+        if highest_percent_idx not in neg_control_sample_idxs:
+            neg_control_sample_idxs[0] = highest_percent_idx
+        if lowest_percent_idx not in neg_control_sample_idxs:
+            neg_control_sample_idxs[-1] = lowest_percent_idx
+    ##############################################################
     neg_control_sample_idxs = neg_control_sample_idxs.tolist()
     neg_control_sample_idxs.sort()
-    print(np.shape(neg_control_sample_idxs))
+    #print(np.shape(neg_control_sample_idxs))
 
     ## subset these IDs
     neg_control_subset_mat = np.array(infile[neg_control_sample_idxs,:])
@@ -762,57 +752,57 @@ if do_sum_neg or args.rho_cutoff==None:
         neg_control_subset_mat[i,:] = neg_control_subset_mat[i,temp_row_shuffle_order]
 
     ## get the spearman_rhos for the shuffled matrix
-    r=no_p_spear(neg_control_subset_mat,neg_control_subset_mat, axis = 1, prop=args.prop)
+    r=no_p_spear(neg_control_subset_mat, neg_control_subset_mat, axis = 1, prop=args.prop)
     r = r[:bin_size,:bin_size]
     print("rho shape",np.shape(r))
-    ## remove self correlations
-    upper_tiangle_indices = np.triu_indices(bin_size,k=1)
     
-    
-    ## subset the negative ones
-    print(r)
-    linear_rho = r[upper_tiangle_indices]
-    print(linear_rho)
-    print(max(linear_rho))
-    print(len(np.where(linear_rho == max(linear_rho))[0]))
-    ## calculate the median absolute deviation
-    #calculate_mad(linear_rho)
+    if single_cutoff:
+        ## remove self correlations
+        upper_tiangle_indices = np.triu_indices(bin_size,k=1)
+        ## this means that we just have a single negative rho cutoff for all genes
+        ## subset the negative ones
+        linear_rho = r[upper_tiangle_indices]
 
-    negative_control_negative_rho_vect = linear_rho[np.where(linear_rho<0)[0]]
-    negative_control_positive_rho_vect = linear_rho[np.where(linear_rho>0)[0]]
-    #calculate_mad(negative_control_negative_rho_vect)
+        negative_control_negative_rho_vect = linear_rho[np.where(linear_rho<0)[0]]
+        negative_control_positive_rho_vect = linear_rho[np.where(linear_rho>0)[0]]
+        #calculate_mad(negative_control_negative_rho_vect)
 
-    ## plot the average Rho per negative correlation
-    print(negative_control_negative_rho_vect)
-    neg_cor_density_x, neg_cor_density_y = get_density(negative_control_negative_rho_vect)
-    pos_cor_density_x, pos_cor_density_y = get_density(negative_control_positive_rho_vect)
-    
-    
-    sum_neg_plot = os.path.dirname(args.adj_list_out)+'/boostrap_neg_cor_rhos.png'
-    plot_one_density(neg_cor_density_x, neg_cor_density_y, sum_neg_plot, label="density of negative correlation null distribution")
-    pos_rho_plot = os.path.dirname(args.adj_list_out)+'/boostrap_pos_cor_rhos.png'
-    plot_one_density(pos_cor_density_x, pos_cor_density_y,pos_rho_plot, label="density of positive correlation null distribution")
+        ## plot the average Rho per negative correlation
+        #print(negative_control_negative_rho_vect)
+        neg_cor_density_x, neg_cor_density_y = get_density(negative_control_negative_rho_vect)
+        pos_cor_density_x, pos_cor_density_y = get_density(negative_control_positive_rho_vect)
+        
+        
+        sum_neg_plot = os.path.dirname(args.adj_list_out)+'/boostrap_neg_cor_rhos.png'
+        plot_one_density(neg_cor_density_x, neg_cor_density_y, sum_neg_plot, label="density of negative correlation null distribution")
+        pos_rho_plot = os.path.dirname(args.adj_list_out)+'/boostrap_pos_cor_rhos.png'
+        plot_one_density(pos_cor_density_x, pos_cor_density_y,pos_rho_plot, label="density of positive correlation null distribution")
 
-    ## plot the distribution of all bootstrap negative control Rhos
-    neg_cor_density_x, neg_cor_density_y = get_density(linear_rho)
-    sum_neg_plot = os.path.dirname(args.adj_list_out)+'/boostrap_cor_rhos.png'
-    plot_one_density(neg_cor_density_x, neg_cor_density_y, sum_neg_plot,label="density of all correlation null distribution")
+        ## plot the distribution of all bootstrap negative control Rhos
+        neg_cor_density_x, neg_cor_density_y = get_density(linear_rho)
+        sum_neg_plot = os.path.dirname(args.adj_list_out)+'/boostrap_cor_rhos.png'
+        plot_one_density(neg_cor_density_x, neg_cor_density_y, sum_neg_plot,label="density of all correlation null distribution")
 
-    print("\n\ncalculating negative correlation FPR cutoff from null distribution:")
-    #neg_cutoff = get_FPR_cutoff(negative_control_negative_rho_vect,FPR)
-    neg_cutoff, FPR = get_Z_cutoff(negative_control_negative_rho_vect)
-    print('\nEmpiric FPR:',FPR)
-    print("\n\ncalculating positive correlation FPR cutoff from null distribution:")
-    #pos_cutoff = get_FPR_cutoff(negative_control_positive_rho_vect,pos_FPR, positive = True)
-    pos_cutoff = get_Z_cutoff(negative_control_positive_rho_vect, z=5.5, positive = True)
-    rho_cutoff = pos_cutoff
-    args.rho_cutoff = pos_cutoff
-    #sys.exit()
+        print("\n\ncalculating negative correlation FPR cutoff from null distribution:")
+        #neg_cutoff = get_FPR_cutoff(negative_control_negative_rho_vect,FPR)
+        #neg_cutoff, FPR = get_Z_cutoff(negative_control_negative_rho_vect, z=3)
+        neg_cutoff, FPR = get_empiric_FPR(negative_control_negative_rho_vect)
+        print('\nEmpiric FPR:',FPR)
+        print("\n\ncalculating positive correlation FPR cutoff from null distribution:")
+        #pos_cutoff = get_FPR_cutoff(negative_control_positive_rho_vect,pos_FPR, positive = True)
+        pos_cutoff = get_Z_cutoff(negative_control_positive_rho_vect, z=5.5, positive = True)
+        rho_cutoff = pos_cutoff
+        args.rho_cutoff = pos_cutoff
+        #sys.exit()
+    else:
+        ##############################################################
+        ## this means that we just have a dynamic negative rho cutoff as determined by gene expression sparsity
+        rho_cutoff_vect, FPR_vect = get_rho_cutoff_vect(r, neg_control_sample_idxs, gene_non_zero_mean, os.path.dirname(args.adj_list_out))
 #################################################################################
     
 ## make the output file
-cmd('rm '+args.adj_list_out)
-cmd('rm '+args.adj_list_out[:-4]+"_dedup.tsv")
+rm(args.adj_list_out)
+rm(args.adj_list_out[:-4]+"_dedup.tsv")
 out_file_name = args.adj_list_out[:-4]+"_dedup.tsv"
 out_file = open(out_file_name, "a")
 
@@ -831,59 +821,114 @@ for rho in rho_files:
     temp_indices, temp_IDs = rho_properties_dict[rho]
     for i in range(0,np.shape(temp_rho_mat)[0]):
         ## go through each line in the rho matrix and log the pertinent indices
-        cur_i_variable = temp_IDs[i]
+        cur_var_1 = temp_IDs[i]
         sig_indices = np.where(np.absolute(temp_rho_mat[i,:])>=args.rho_cutoff)[0]
         sig_IDs = [temp_IDs[j] for j in sig_indices]
         sig_rhos = temp_rho_mat[i,sig_indices]
 
         ## go through the significant indicies
         for q in range(0,len(sig_IDs)):
-            if sig_IDs[q] != cur_i_variable:
-                if ID_hash[cur_i_variable] > ID_hash[sig_IDs[q]]:
-                    ## check if variable pair has been done already
-                    temp_relationships = all_vars_adj_dict[cur_i_variable]
-                    var2_idx = ID_hash[sig_IDs[q]]
-                    ## check if it has been done (ie: currently set to zero.)
-                    if temp_relationships[var2_idx] == 0:
-                        temp_relationships[var2_idx]+=1## this is the indicator that we've doen it already
-                        all_vars_adj_dict[cur_i_variable] = temp_relationships
-                        #temp_out_line = [str(cur_i_variable), str(sig_IDs[q]), 'NA', 'NA','NA','NA',str(sig_rhos[q])]
-                        temp_out_line = [str(cur_i_variable), str(sig_IDs[q]), str(sig_rhos[q])]
-                        out_file.write('\t'.join(temp_out_line)+'\n')
-                        ## also log the positive correlations
-                        num_pos_cor_dict[str(cur_i_variable)] = 1 + num_pos_cor_dict[str(cur_i_variable)] 
-                        num_pos_cor_dict[str(sig_IDs[q])] = 1 + num_pos_cor_dict[str(sig_IDs[q])] 
+            if ID_hash[cur_var_1] > ID_hash[sig_IDs[q]]:
+                ## check if variable pair has been done already
+                temp_relationships = all_vars_adj_dict[cur_var_1]
+                var2_idx = ID_hash[sig_IDs[q]]
+                ## check if it has been done (ie: currently set to zero.)
+                if temp_relationships[var2_idx] == 0:
+                    temp_relationships[var2_idx]+=1## this is the indicator that we've doen it already
+                    all_vars_adj_dict[cur_var_1] = temp_relationships
+                    #temp_out_line = [str(cur_var_1), str(sig_IDs[q]), 'NA', 'NA','NA','NA',str(sig_rhos[q])]
+                    temp_out_line = [str(cur_var_1), str(sig_IDs[q]), str(sig_rhos[q])]
+                    out_file.write('\t'.join(temp_out_line)+'\n')
+                    ## also log the positive correlations
+                    num_pos_cor_dict[str(cur_var_1)] = 1 + num_pos_cor_dict[str(cur_var_1)] 
+                    num_pos_cor_dict[str(sig_IDs[q])] = 1 + num_pos_cor_dict[str(sig_IDs[q])] 
         
         if do_sum_neg:
             ## do the same for all of the negative correlations, regardless of magnitude
-            total_negative_indices = np.where(temp_rho_mat[i,:]<0)[0]
-            neg_IDs = [temp_IDs[j] for j in total_negative_indices]
+            total_negative_indices = np.where(temp_rho_mat[i,:]<0)[0] ## negative gobal indices relative to current gene
+            neg_IDs = [temp_IDs[j] for j in total_negative_indices] ## other variable's names
             neg_rhos = temp_rho_mat[i,total_negative_indices]
+            cur_var_1_index = ID_hash[cur_var_1]
             for q in range(0,len(neg_rhos)):
-                if ID_hash[cur_i_variable] > ID_hash[neg_IDs[q]]:
-
+                cur_var_2 = neg_IDs[q]
+                cur_var_2_index = ID_hash[neg_IDs[q]]
+                if cur_var_1_index < cur_var_2_index:
+                    if (cur_var_1 =="Gene1" or cur_var_1 == "Gene2" or cur_var_2 =="Gene1" or cur_var_2 == "Gene2") and (len(cur_var_1)<=7 and len(cur_var_2)<=7):
+                        local_verbose = False
+                        #print("cur_var_1",cur_var_1,"\tvs\t",cur_var_2)
+                    else:
+                        local_verbose = False
                     ## check if variable pair has been done already
-                    temp_relationships = total_neg_vars_adj_dict[cur_i_variable]
-                    var2_idx = ID_hash[neg_IDs[q]]
-                    if temp_relationships[var2_idx] == 0:
-                        ## total number of Rho <0
-                        temp_relationships[var2_idx]+=1
-                        total_neg_vars_adj_dict[cur_i_variable] = temp_relationships
-                        sum_neg[ID_hash[cur_i_variable]][3]+=1
-                        sum_neg[ID_hash[neg_IDs[q]]][3]+=1
-
+                    temp_relationships = total_neg_vars_adj_dict[cur_var_1] ## pull the boolean cateloge of what's been done
+                    temp_relationships2 = total_neg_vars_adj_dict[cur_var_2] ## pull the boolean cateloge of what's been done
+                    if temp_relationships[cur_var_2_index] == False and temp_relationships2[cur_var_1_index] == False: ## meaning we haven't check this pair yet
+                        ## housekeeping to keep track of which variables we've already done
+                        temp_relationships[cur_var_2_index] = True ## first log that we have now done the comparison
+                        temp_relationships2[cur_var_1_index] = True
+                        total_neg_vars_adj_dict[cur_var_1] = temp_relationships ## set it again
+                        total_neg_vars_adj_dict[cur_var_2] = temp_relationships2
+                        ########################
+                        ## now log the results
+                        sum_neg[cur_var_1_index][3]+=1 ## this is the total number of negative correlations
+                        sum_neg[cur_var_2_index][3]+=1 ## this is the total number of negative correlations
                         ## then if it's actually less than the simulation determined cutoff, log it for that as well
-                        if neg_rhos[q]<neg_cutoff:# and neg_rhos[q]>-1 :
-                            sum_neg[ID_hash[cur_i_variable]][1]+=neg_rhos[q]
-                            sum_neg[ID_hash[neg_IDs[q]]][1]+=neg_rhos[q]
+                        #######################################################
+                        ## 11/14/2020 this is where we can change the cutoff to be dynamic depending on the percent express
+                        #######################################################
+                        if single_cutoff:
+                            if neg_rhos[q] < neg_cutoff:# and neg_rhos[q]>-1 :
+                                sum_neg[cur_var_1_index][1]+=neg_rhos[q] ## this is the number of "significant" negative correlations
+                                sum_neg[cur_var_2_index][1]+=neg_rhos[q] ## this is the number of "significant" negative correlations
 
-                            ## total number < auto-determined Rho cutoff from bootstrap shuffled negative correlations
-                            sum_neg[ID_hash[cur_i_variable]][2]+=1
-                            sum_neg[ID_hash[neg_IDs[q]]][2]+=1
+                                ## total number < auto-determined Rho cutoff from bootstrap shuffled negative correlations
+                                sum_neg[cur_var_1_index][2]+=1
+                                sum_neg[cur_var_2_index][2]+=1
+                                if local_verbose:
+                                    print("\trho:",neg_rhos[q])
+                                    print("\ttotal_neg_vars_adj_dict[cur_var_1]",total_neg_vars_adj_dict[cur_var_1])
+                                    print("\ttotal_neg_vars_adj_dict[cur_var_2]", total_neg_vars_adj_dict[cur_var_2])
+                                    print("\tsum_neg[cur_var_1_index][3]",sum_neg[cur_var_1_index][3])
+                                    print("\tsum_neg[cur_var_2_index][3]",sum_neg[cur_var_2_index][3])
+                                    print("\tsum_neg[cur_var_1_index][1]",sum_neg[cur_var_1_index][1])
+                                    print("\tsum_neg[cur_var_2_index][1]",sum_neg[cur_var_2_index][1])
+                                    print("\tsum_neg[cur_var_1_index][2]",sum_neg[cur_var_1_index][2])
+                                    print("\tsum_neg[cur_var_2_index][2]",sum_neg[cur_var_2_index][2])
 
+                        else:
+                            ## if it's less than the i variable's cutoff
+                            if neg_rhos[q] < neg_cutoff_vect[cur_var_1_index]:
+                                sum_neg[cur_var_1_index][1]+=neg_rhos[q] ## this is the number of "significant" negative correlations
+                                ## total number < auto-determined Rho cutoff from bootstrap shuffled negative correlations
+                                sum_neg[cur_var_1_index][2]+=1
+                            if neg_rhos[q] < neg_cutoff_vect[cur_var_2_index]:
+                                sum_neg[cur_var_2_index][1]+=neg_rhos[q] ## this is the sum of all of the negative rhos - turned out to be less useful
+                                ## total number < auto-determined Rho cutoff from bootstrap shuffled negative correlations
+                                sum_neg[cur_var_2_index][2]+=1 ## this is the number of "significant" negative correlations
 
 out_file.close()
 
+
+def pass_cutoff(above, total, num_pos_cor, num_pos_cor_cutoff = 10):
+    global FPR,FPR_multiple, expected_x, expected_y
+    ## the FPR variable is really the denominator of the actuall false positive rate. 
+    ## This means that 1 in every FPR negative correlations should exceed the cutoff
+    ## purely by chance.
+    ## So , 
+    this_genes_number_of_expected = total*(1/FPR)
+    this_genes_multiple_of_expected_cutoff = this_genes_number_of_expected * FPR_multiple
+    # print("\tabove:", above,"total:",total, "FPR:",FPR, "FPR_multiple:",FPR_multiple)
+    # print("\tthis_genes_number_of_expected:",this_genes_number_of_expected)
+    # print("\tthis_genes_multiple_of_expected_cutoff:",this_genes_multiple_of_expected_cutoff)
+    # print("\tpassing?:",above > this_genes_multiple_of_expected_cutoff)
+    if above < 1:#10:
+        return("False")
+    #elif total/above < (FPR/FPR_multiple) and num_pos_cor >= num_pos_cor_cutoff:
+    elif (above > this_genes_multiple_of_expected_cutoff) and num_pos_cor >= num_pos_cor_cutoff:
+        return("True")
+    else:
+        return("False")
+
+
 if do_sum_neg:
     ## sum up the negative correlations for the bootstrap shuffled negative control
     negative_control_sum_neg = []
@@ -908,7 +953,7 @@ if do_sum_neg:
 
     ######################################################################################
     ## process the sum negative table into a vector so that we can do interesting things with it
-    print(sum_neg[:10])
+    #print(sum_neg[:10])
     sum_neg_vect = np.zeros(len(sum_neg),dtype = float)
     sum_neg_count = np.zeros(len(sum_neg),dtype = float)
     sum_neg_count_total = np.zeros(len(sum_neg),dtype = float)
@@ -930,7 +975,7 @@ if do_sum_neg:
         sum_neg[i][4]=count_above_below_ratio[i]
 
     avg_neg = sum_neg_vect/sum_neg_count
-    print(sum_neg_vect[:10])
+    #print(sum_neg_vect[:10])
 
 
     ## neg count
@@ -942,35 +987,17 @@ if do_sum_neg:
     expected_x = np.log2(expected_x)
     expected_x = expected_x[1:]
     expected_y = expected_y[1:]
-    print(expected_y)
-    print(expected_x)
+    #print(expected_y)
+    #print(expected_x)
     #sys.exit()
     sum_neg_count = np.log2(sum_neg_count+1)
     #med,med_dev,cutoff = calculate_mad(sum_neg_count,mean=False)
 
     ## append whether or not they passed the cutoff to the summary matrix
-    def pass_cutoff(above, total, num_pos_cor, num_pos_cor_cutoff = 10):
-        global FPR,FPR_multiple, expected_x, expected_y
-        ## find the nearest for total (which corresponds to expected_y)
-        lowest_index = np.argmin(abs(total - expected_y))
-        ## the corresponding y index is the cutoff value
-        temp_cutoff = expected_x[lowest_index]
-        # if above < 10:
-        #     return("False")
-        # elif above > temp_cutoff:
-        #     return("True")
-        # else:
-        #     return("False")
-        if above < 1:#10:
-            return("False")
-        elif total/above < (FPR/FPR_multiple) and num_pos_cor >= num_pos_cor_cutoff:
-            return("True")
-        else:
-            return("False")
-
     #[ID, sum_negative_rho, count_below_threshold, total_number_negative, (count_below_threshold+1)/(count_above_threshold +1)]
     included_indices = []
     for i in range(0,len(sum_neg)):
+        #print(sum_neg[i][0])
         sum_neg[i].append(pass_cutoff(sum_neg[i][2],sum_neg[i][3], num_pos_cor_dict[sum_neg[i][0]], num_pos_cor_cutoff = args.num_pos_cor_cutoff))
         if eval(sum_neg[i][-1]):
             included_indices.append(i)
@@ -1017,18 +1044,4 @@ if do_sum_neg:
 
 
 
-################################################################################
-
-#cmd('remove_dups.py -infile '+args.adj_list_out)
-
-
-
-
-
-
-
-
-
-
-
 
