@@ -23,7 +23,7 @@ import numpy as np
 from scipy.stats import rankdata
 from copy import deepcopy
 import h5py
-
+from math import floor
 from scipy.stats import mstats_basic, rankdata
 
 #from . import distributions
@@ -110,6 +110,10 @@ def make_dense(b):
     return(b)
 
 
+def dense_rank(a):
+    return(rankdata(a,method="dense"))
+    
+
 def no_p_spear(a, b=None, axis=0, nan_policy='propagate', rm_nans_by_col = False, prop = False):
     a=make_dense(a)
     b=make_dense(b)
@@ -129,7 +133,8 @@ def no_p_spear(a, b=None, axis=0, nan_policy='propagate', rm_nans_by_col = False
     
     if a.size <= 1:
         return SpearmanrResult(np.nan, np.nan)
-    ar = np.apply_along_axis(rankdata, axisout, a)
+    #ar = np.apply_along_axis(rankdata, axisout, a)
+    ar = np.apply_along_axis(dense_rank, axisout, a)
     
     br = None
     if b is not None:
@@ -141,7 +146,8 @@ def no_p_spear(a, b=None, axis=0, nan_policy='propagate', rm_nans_by_col = False
             b = ma.masked_invalid(b)
             return no_p_spear(a, b, axis)
         
-        br = np.apply_along_axis(rankdata, axisout, b)
+        #br = np.apply_along_axis(rankdata, axisout, b)
+        br = np.apply_along_axis(dense_rank, axisout, b)
     n = a.shape[axisout]
     if not prop:
         rs = np.corrcoef(ar, br, rowvar=axisout)
@@ -506,3 +512,35 @@ def get_Z_cutoff(vect, z=4.5,positive = False):
         return(cutoff, FPR)
     else:
         return(cutoff)
+
+
+
+def get_empiric_FPR(vect, FPR=1000, positive = False):
+    ## this is for a half distribution, so mult by -1 to make it 'normal'
+    if positive:
+        vect = vect * -1
+    vect_original = np.array(deepcopy(vect))
+    keep_not_nan = np.logical_not(np.isnan(vect_original))
+    keep_not_zero = vect_original!=0
+    keep_not_one = vect_original!=1
+    keep_not_neg_one = vect_original!=-1
+    keep = keep_not_nan * keep_not_zero * keep_not_one * keep_not_neg_one
+    ## remove zeros, 1s and -1s
+    vect_original = vect_original[keep]
+    ## either this
+    target_index = floor(vect_original.shape[0]/FPR)
+    print("target index:",target_index)
+    #if not positive:
+    #    vect_original = vect_original * -1
+    sort_order = np.sort(vect_original)
+    cutoff = sort_order[target_index]
+    print('cutoff:',cutoff)
+    if not positive:
+        num_sig = np.sum(np.array(vect_original<cutoff,dtype = bool))
+        total_num = vect_original.shape[0]
+        print(num_sig, "/", total_num, 'significant')
+        FPR = max([1,total_num])/num_sig
+        return(cutoff, FPR)
+    else:
+        return(cutoff * -1)
+
